import { ZhCloseButton } from '@/components/Button'
import { iFile, iOSSData } from '@/types/file'
import { getSignature } from '@/utils/common/api'
import consts from '@/utils/consts'
import { InboxOutlined } from '@ant-design/icons'
import { Button, message, Modal, Upload } from 'antd'
import { ModalProps } from 'antd/lib/modal'
import { UploadChangeParam } from 'antd/lib/upload'
import { UploadFile } from 'antd/lib/upload/interface'
import React, { useEffect, useState } from 'react'
const { Dragger } = Upload

interface iUploadModalProps extends ModalProps {
  onCreate: (files: iFile[]) => void
  onShow: (visible: boolean) => void
}
const UploadModal: React.FC<iUploadModalProps> = (props) => {
  const { onCreate, onShow, ...otherProps } = props
  const [ files, setFiles ] = useState<iFile[]>([])
  const [ fileList, setFileList ] = useState<Array<UploadFile>>([])
  const [ OSSData, setOssData ] = useState<iOSSData>({
    dir: '',
    expire: '',
    host: '',
    accessId: '',
    policy: '',
    signature: ''
  })
  useEffect(() => {
    if (props.visible) {
      // 初始化时要将上次上次的文件清除掉
      setFileList([])
      setFiles([])
      initOssData()
    }
  }, [ props.visible ])
  // 上传文件改变时的状态
  const onChange = (info: UploadChangeParam) => {
    const { status } = info.file
    if (status === 'done') {
      const newFiles = info.fileList
        .filter(file => file.status !== 'error')
        .map(item => ({ createTime: new Date(), filepath: item.url, filename: item.name }))
      setFiles(newFiles)
    } else if (status === 'error') {
      // console.log(info)

      // message.error(`${info.file.name} 上传失败`)
    }
    setFileList(info.fileList)

  }
  // 初始化、获取签名
  const initOssData = async () => {
    const { code = -1, data = {}, msg = '获取签名失败' } = await getSignature()
    if (code === consts.RET_CODE.SUCCESS) {
      setOssData({ ...OSSData, ...data })
    } else {
      message.error(msg)

    }
  }

  // 额外参数、oss签名
  const getExtraData = (file: UploadFile) => {
    return {
      key: file.url,
      OSSAccessKeyId: OSSData.accessId,
      policy: OSSData.policy,
      signature: OSSData.signature
    }
  }

  // 上传前的回调
  const beforeUpload = async (file: any) => {
  const { UPLOAD_LIMIT } = consts
    const isLt30M = file.size / 1024 / 1024 < UPLOAD_LIMIT

    if (!isLt30M) {
      file.status = 'error'
      message.error("上传附件大小限制在30MB")
      return false
    }
    const expire = parseInt(OSSData.expire) * 1000
    if (expire < Date.now()) {
      try {
        await initOssData()
      } catch (error) {
        file.status = 'error'
        message.error('获取签名失败，请联系管理员')
        return false
      }
    }

    const reg = new RegExp(consts.UPLOAD_WHITE, 'i')
    if (!reg.test(file.name)) {
      file.status = 'error'
      message.error('不支持该类型文件')
      return false
    }

    file.url = OSSData.dir + file.name
    return Promise.resolve(file)
  }


  // 移除文件
  const onRemove = (file: any) => {
    const { uid = '', name = '' } = file
    const newFiles = files.filter(item => item.filename !== name)
    const newFileList = fileList.filter(item => item.uid !== uid)
    setFileList(newFileList)
    setFiles(newFiles)
  }

  return (
    <Modal
      {...otherProps}
      title="上传附件"
      getContainer={false}
      footer={
        <div className="pi-flex-row pi-justify-end">
          <ZhCloseButton onClick={() => onShow(false)} size="small" className="pi-mg-right-5">关闭</ZhCloseButton>
          <Button type="primary" size="small" onClick={() => onCreate(files)}>确认</Button>
        </div>}
    >
      <Dragger
        name="file"
        fileList={fileList}
        action={OSSData.host}
        multiple={true}
        onChange={onChange}
        beforeUpload={beforeUpload}
        data={getExtraData}
        onRemove={onRemove}
        onPreview={(file) => {
          const status = file.status
          if (status && status === 'success' || status === 'done') {
            window.open(consts.OSS_PATH.REVIEW + file.url)
          }
        }}

      >
        <p className="ant-upload-drag-icon">
          <InboxOutlined />
        </p>
        <p className="ant-upload-text">把文件拖入指定区域，完成上传，同样支持点击上传。</p>
        <p className="ant-upload-hint">
          支持单文件/多文件上传。附件大小最大为30MB
        </p>
      </Dragger>
    </Modal>
  )
}

export default UploadModal
