import React, { useContext, useState, useEffect, useRef } from 'react'
import { Input, Form } from 'antd'
import { FormInstance } from 'antd/lib/form'
import { ContractTree } from '@/types/contract'

const EditableContext = React.createContext<FormInstance<any> | null>(null)

interface EditableRowProps {
  index: number;
}

const EditableRow: React.FC<EditableRowProps> = ({ index, ...props }) => {
  const [ form ] = Form.useForm()
  return (
    <Form form={form} component={false}>
      <EditableContext.Provider value={form}>
        <tr {...props} />
      </EditableContext.Provider>
    </Form>
  )
}

interface EditableCellProps {
  title: React.ReactNode;
  editable: boolean;
  children: React.ReactNode;
  dataIndex: keyof ContractTree;
  record: ContractTree;
  handleSave: (dataIndex: keyof ContractTree, value: string, record: ContractTree) => void;
}

const EditableCell: React.FC<EditableCellProps> = ({
  title,
  editable,
  children,
  dataIndex,
  record,
  handleSave,
  ...restProps
}) => {
  const [ editing, setEditing ] = useState(false)
  const inputRef = useRef<Input>(null)
  const form = useContext(EditableContext)!


  useEffect(() => {
    if (editing) {
      inputRef.current!.focus()
    }
  }, [ editing ])

  const toggleEdit = () => {
    setEditing(!editing)
    if (dataIndex === 'code') {
      form.setFieldsValue({ [dataIndex]: record.serial })
    } else {
      form.setFieldsValue({ [dataIndex]: record[dataIndex] })
    }
  }

  const save = async () => {
    try {
      const values = await form.validateFields()

      toggleEdit()
      handleSave(dataIndex, values[dataIndex], record)
    } catch (errInfo) {
      console.log('Save failed:', errInfo)
    }
  }

  let childNode = children

  if (editable && record.depth !== 1) {
    childNode = editing ? (
      <Form.Item
        style={{ margin: 0 }}
        name={dataIndex}
      >
        {
          dataIndex === 'code' ?
          <Input size="small" ref={inputRef} onPressEnter={save} onBlur={save} addonBefore={record.attribution} autoComplete="off"/>
          :
          <Input size="small" ref={inputRef} onPressEnter={save} onBlur={save} autoComplete="off"/>

        }
      </Form.Item>
    ) : (
      <div className="editable-cell-value-wrap" style={{ paddingRight: 24 }} onDoubleClick={toggleEdit}>
        {children}
      </div>
    )
  }

  return <td {...restProps}>{childNode}</td>
}


export {
  EditableRow,
  EditableCell
}
