import { useTabHeighAuto } from '@/hooks/tabHooks'
import { contractStore } from '@/store/mobx'
import { ContractType } from '@/store/mobx/contract'
import { apiDelFile, apiGetFileList } from '@/utils/common/api'
import consts from '@/utils/consts'
import { dayjsFormat } from '@/utils/util'
import { Popconfirm } from 'antd'
import Table, { ColumnsType } from 'antd/lib/table'
import { observer } from 'mobx-react'
import React, { useEffect, useState } from 'react'
import styles from './index.module.scss'

interface iFileState {
  id: string
  filename: string
  filepath: string
  accountName: string
  createTime: string
}

interface FiltTable {
  data: iFileState[]
  total: number
  pageNo: number
  pageSize: number
}

const File: React.FC<{ type: 'income' | 'expenditure' }> = ({ type }) => {
  const [ state, setState ] = useState<FiltTable>({ data: [], total: 0, pageNo: 1, pageSize: 8 })
  const [ id, setId ] = useState<string>('')
  const y = useTabHeighAuto()
  useEffect(() => {
    if (contractStore.contract.id) {
      if (contractStore.contract.id !== id) {
        setId(contractStore.contract.id)
        initData()
      } else if (contractStore.activeKey && contractStore.activeKey === '3') {
        initData(state.pageNo, state.pageSize)
      }
    } else {
      setState({ ...state, data: [], total: 0 })
      setId('')
    }
  }, [ contractStore.contract.id, contractStore.activeKey ])

  const initData = async (pageNo: number = 1, pageSize: number = 8) => {
    const { code = -1, data = [], total = 0 } = await apiGetFileList(type === ContractType.INCOME ? consts.DATA_TYPE.INCOME : consts.DATA_TYPE.EXPENDITURE, contractStore.contract.id, pageNo, pageSize)
    if (code === consts.RET_CODE.SUCCESS) {
      setState({ ...state, data, total, pageNo, pageSize })
      console.log(state.pageNo)

      contractStore.activeKey && (contractStore.changeActiveKey(''))
    }
  }

  const deleteFile = async (id: string) => {
    const { code = -1 } = await apiDelFile(id)
    if (code === consts.RET_CODE.SUCCESS) {
      const newData = state.data.filter((file: iFileState) => file.id !== id)
      setState({ ...state, data: newData })
    }
  }
  const columns: ColumnsType<iFileState> = [
    {
      dataIndex: 'sort',
      // eslint-disable-next-line react/display-name
      render: (_: any, record: iFileState, idx: number) => <span>{((state.pageNo - 1) * state.pageSize) + idx + 1}</span>,
      width: '5%'
    },
    {
      title: '名称',
      dataIndex: 'filename',
      // eslint-disable-next-line react/display-name
      render: (text: string, record: iFileState) => <a href={consts.OSS_PATH.REVIEW + record.filepath} target="_blank" rel="noopener noreferrer">{text}</a>,
      width: '50%'
    },
    {
      title: '上传者',
      dataIndex: 'accountName',
      width: '10%'
    },
    {
      title: '上传时间',
      dataIndex: 'createTime',
      // eslint-disable-next-line react/display-name
      render: (time: string) => <span>{dayjsFormat(time, 'YYYY-MM-DD')}</span>,
      width: '20%'
    },
    {
      title: '操作',
      dataIndex: 'opreate',
      // eslint-disable-next-line react/display-name
      render: (_: string, record: iFileState) => {
        return <div>
          <a className="pi-mg-right-5" download href={consts.OSS_PATH.DOWNLOAD + record.filepath}>下载</a>
          <Popconfirm title="确认删除?" cancelText="取消" okText="确认" okButtonProps={{ danger: true }} onConfirm={() => deleteFile(record.id)}>
            <span className="pi-link-red pi-mg-left-5">删除</span>
          </Popconfirm>
        </div>
      },
      width: '15%'
    }
  ]
  return (
    <div className={styles.contractFileTab}>
      <Table
        dataSource={state.data}
        columns={columns}
        bordered
        rowKey={record => record.id}
        scroll={{ y }}
        pagination={{
          hideOnSinglePage: true,
          size: "small",
          defaultPageSize: 8,
          showSizeChanger:false,
          onChange: (page, pageSize) => initData(page, pageSize),
          total: state.total
        }}
      />

    </div>
  )
}

export default observer(File)
