import DatePicker from '@/components/DatePicker'
import FileModal from '@/components/FileModal'
import MoneyInput from '@/components/MoneyInput'
import { useTabHeighAuto } from '@/hooks/tabHooks'
import { commonStore, contractStore } from '@/store/mobx'
import { ContractType } from '@/store/mobx/contract'
import { iReceivableState, iEditableCellProps } from '@/types/contract'
import { iFileModal } from '@/types/file'
import { contractConsts } from '@/utils/common/constStatus'
import consts from '@/utils/consts'
import { dayjsFormat, formatMoney } from '@/utils/util'
import { DisconnectOutlined } from '@ant-design/icons'
import { Form, Input, Popconfirm, Table, Select } from 'antd'
import BigNumber from 'bignumber.js'
// import locale from 'antd/es/date-picker/locale/zh_CN'
import dayjs from 'dayjs'
import { observer } from 'mobx-react'
import React, { useEffect, useState } from 'react'
import { apiDelReturn, apiGetReturns, apiUpdateReturn } from './api'
const { Option } = Select
import styles from './index.module.scss'
const EditableCell: React.FC<iEditableCellProps> = ({
  editing,
  dataIndex,
  title,
  cellType,
  children,
  record,
  index,
  ...restProps
}) => {

  let cellNode: React.ReactNode
  switch (cellType) {
    case 'text':
      cellNode = <Input size="small" allowClear autoComplete="off" />
      break
    case 'DatePicker':
      cellNode = <DatePicker size="small" allowClear />
      break
    case 'MoneyInput':
      cellNode = <MoneyInput></MoneyInput>
      break
    case 'Select':
      cellNode = <Select size="small">{commonStore.returnWayOptions.map(item => (<Option key={item} value={item}>{item}</Option>))}</Select>
      break
    default:
      break
  }

  // const isDate = useMemo(() => {
  //   return dataIndex === 'createTime' || dataIndex === 'time'
  // }, [ dataIndex ])


  return (
    <td {...restProps}>
      {editing ? (
        cellType === 'MoneyInput' ?
        <Form.Item name={dataIndex} rules={[ { required: true, message: '请输入合同金额' } ]} className={styles.MoneyFormItem}>
          {cellNode}
        </Form.Item>:
        <Form.Item name={dataIndex} style={{ margin: 0 }} rules={[ dataIndex === 'remarks' ? {} : { required: true, message: `请${cellType === 'text' ? '输入' : '选择'}${title}!` } ]}>
          {cellNode}
        </Form.Item>
      ) : (
          children
        )}
    </td>
  )
}

interface ReceivableProps {
  updateTreeAndContract: () => Promise<void>
  type: 'income' | 'expenditure'
}
const Receivable: React.FC<ReceivableProps> = ({ updateTreeAndContract, type }) => {
  const y = useTabHeighAuto()
  const [ form ] = Form.useForm()
  const [ data, setData ] = useState<Array<iReceivableState>>([])
  const [ rowId, setRowId ] = useState<string>('')
  const [ page, setPage ] = useState(1)
  const [ fileModal, setFileModal ] = useState<iFileModal>({
    visible: false,
    dataType: consts.DATA_TYPE[type === ContractType.INCOME ? 'CONTRACT_RETURN' : 'CONTRACT_PAID'],
    dataId: ''
  })
  const [ editingKey, setEditingKey ] = useState<string>('')

  const delConfirm = async (id: string, contractsId: string, bidsectionId: string) => {
    const { code = -1 } = await apiDelReturn(type, id, contractsId, bidsectionId)
    if (code === consts.RET_CODE.SUCCESS) {
      const newData = data.filter(item => item.id !== id)
      setData(newData)
      updateTreeAndContract()
    }
  }

  useEffect(() => {

    !commonStore.returnWayOptions.length && (commonStore.queryWayOptions())
    if (contractStore.contract.id) {
      if (contractStore.contract.id !== rowId) {
        setRowId(contractStore.contract.id)
        initData()
      } else if (contractStore.activeKey && contractStore.activeKey === '2') {

        initData()
      }
    } else {
      setRowId('')
      // 项目节的点击，数据为空数组
      setData([])
    }

  }, [ contractStore.activeKey, contractStore.contract.id ])

  const initData = async () => {
    const { code = -1, data = [] } = await apiGetReturns(type, contractStore.contract.id, contractStore.contract.bidsectionId)
    if (code === consts.RET_CODE.SUCCESS) {
      setData(data)
      contractStore.activeKey && (contractStore.changeActiveKey(''))
    }
  }
  const save = async (key: React.Key) => {
    try {
      const row = await form.validateFields()

      const newData = [ ...data ]
      const index = newData.findIndex(item => key === item.id)
      if (index > -1) {
        const item = newData[index]
        const payload = { ...row, time: dayjsFormat(row.time, 'YYYY-MM-DD'), createTime: dayjsFormat(row.createTime, 'YYYY-MM-DD'), id: item.id, bidsectionId: item.bidsectionId, contractsId: item.contractsId, price: parseFloat(row.price) }
        const { code = -1 } = await apiUpdateReturn(type, payload)
        if (code === consts.RET_CODE.SUCCESS) {
          newData.splice(index, 1, {
            ...item,
            ...row
          })
          setData(newData)
          await updateTreeAndContract()
        }
      }
      setEditingKey('')
    } catch (errInfo) {
      console.log('Validate Failed:', errInfo)
    }
  }

  const isEditing = (record: iReceivableState) => record.id === editingKey
  const columns = [
    {
      dataIndex: 'sort',
      width: '5%',
      // eslint-disable-next-line react/display-name
      render: (_: any, record: any, index: number) => {
        return <span>{((page - 1) * 8 ) +index + 1}</span>
      }
    },
    {
      title: type === ContractType.INCOME ? '回款日期' : '支付日期',
      dataIndex: 'time',
      editable: true,
      width: '12%',
      // eslint-disable-next-line react/display-name
      render: (text: string) => <span>{dayjsFormat(text, 'YYYY-MM-DD')}</span>
    },
    {
      title: type === ContractType.INCOME ? '回款金额' : '支付金额',
      dataIndex: 'price',
      editable: true,
      width: '12%',
      // eslint-disable-next-line react/display-name
      render: (text: string) => <span className="pi-text-right pi-width-100P">{formatMoney(text)}</span>
    },
    {
      title: type === ContractType.INCOME ? '回款方式' : '支付方式',
      dataIndex: 'way',
      editable: true,
      width: '12%'
    },
    {
      title: '创建人',
      dataIndex: 'createUser',
      editable: false,
      width: '12%'
    },
    {
      title: '创建时间',
      dataIndex: 'createTime',
      editable: false,
      width: '12%',
      // eslint-disable-next-line react/display-name
      render: (text: any) => <span>{dayjsFormat(text, 'YYYY-MM-DD')}</span>
    },
    {
      title: '备注',
      dataIndex: 'remarks',
      editable: true,
      width: '12%'
    },
    {
      title: '附件',
      dataIndex: 'fileCounts',
      // eslint-disable-next-line react/display-name
      render: (text: number, record: iReceivableState) => <span className="pi-pointer" onClick={() => setFileModal({ ...fileModal, dataId: record.id, visible: true })}><DisconnectOutlined style={{ color: '#007bff' }} /> {text}</span>
    },
    {
      title: '操作',
      dataIndex: 'opreate',
      width: '10%',
      // eslint-disable-next-line react/display-name
      render: (text: any, record: iReceivableState) => {
        const editable = isEditing(record)
        return contractStore.contract.status !== contractConsts.status.closed ?
          <div>
            {
              editable ?
                (<><span className="pi-link-blue pi-mg-right-5" onClick={() => save(record.id)}>保存</span><span className="pi-link-blue" onClick={() => setEditingKey('')}>取消</span></>)
                :
                <span className="pi-link-blue" onClick={() => handleRowEdit(record.id)}>编辑</span>
            }
            <Popconfirm title="确认删除?" cancelText="取消" okText="确认" okButtonProps={{ danger: true }} onConfirm={() => delConfirm(record.id, record.contractsId, record.bidsectionId)}>
              <span className="pi-link-red pi-mg-left-5">删除</span>
            </Popconfirm>
          </div>
          : null
      }
    }
  ]

  const handleRowEdit = (id: string) => {
    const formItem = data.find(item => item.id === id)
    if (formItem) {
      const newFormItem = { ...formItem, createTime: dayjs(formItem.createTime), time: dayjs(formItem.time) }
      form.setFieldsValue(newFormItem)
      setEditingKey(id)

    }
  }

  const cancel = (pageNo: number) => {
    setPage(pageNo)
    setEditingKey('')
  }


  const mergedColumns = columns.map(col => {
    if (!col.editable) {
      return col
    }
    let cellType = 'text'
    switch (col.dataIndex) {
      case 'createTime':
        cellType = 'DatePicker'
        break
      case 'time':
        cellType = 'DatePicker'
        break
      case 'way':
        cellType = 'Select'
        break
      case 'price':
        cellType = 'MoneyInput'
        break
      default:
        break
    }
    return {
      ...col,
      onCell: (record: iReceivableState) => ({
        record,
        cellType,
        dataIndex: col.dataIndex,
        title: col.title,
        editing: isEditing(record)
      })
    }
  })

  return (
    <div className={styles.contractProveTab}>
      <Form form={form} component={false}>
        <Table
          components={{ body: { cell: EditableCell } }}
          dataSource={data}
          columns={mergedColumns}
          bordered
          rowClassName="editable-row"
          scroll={{ y }}
          pagination={{ onChange: cancel, size: "small", pageSize: 8 }}
          rowKey={record => record.id}
        />
      </Form>
      <FileModal
        visible={fileModal.visible}
        dataType={fileModal.dataType}
        dataId={fileModal.dataId}
        onCancel={() => setFileModal({ ...fileModal, visible: false })}
        showUpload={true}
        uploadCallBack={() => initData()}
      />
    </div>
  )
}
export default observer(Receivable)
