import { iModalProps, iTenderFormValue, TenderTree } from '@/types/tender'
import { Form, Input, message, Modal, TreeSelect } from 'antd'
import React, { useEffect, useState } from 'react'

interface Option {
  value: string | number;
  title: React.ReactNode;
  disabled: boolean;
  children?: Option[];
}


interface iModalFormProps {
  treeObj: TenderTree
  modalObj: iModalProps
  onCreate: (values: iTenderFormValue, type: string) => void
  onCancel: () => void
}

const ModalForm: React.FC<iModalFormProps> = ({
  treeObj,
  onCreate,
  onCancel,
  modalObj: { visible, confirmLoading, type, id, name, isFolder }
}) => {
  const [ form ] = Form.useForm()

  const modalObj = {
    tender: {
      title: '添加新标段',
      nameTitle: '标段名称',
      namePlaceholder: '输入标段名称',
      okText: '确认添加'
    },
    folder: {
      title: '新建文件夹',
      nameTitle: '文件夹名称',
      namePlaceholder: '输入文件夹名称',
      okText: '确认添加'
    },
    root: {
      title: '新建文件夹',
      nameTitle: '文件夹名称',
      namePlaceholder: '输入文件夹名称',
      okText: '确认添加'
    },
    rename: {
      title: '重命名',
      nameTitle: '名称',
      namePlaceholder: '输入新的名称',
      okText: '确认修改'
    },
    delFolder: {
      title: '删除文件夹',
      nameTitle: '',
      okText: '确认删除'
    },
    delTender: {
      title: '删除标段',
      nameTitle: '',
      okText: '确认删除'
    },
    move: {
      title: '移动',
      nameTitle: '移动至',
      namePlaceholder: '选择文件夹父级目录',
      okText: '确认修改'
    }
  }

  const [ cascader, setCascader ] = useState<Option[]>([])
  const [ showNameInput, setShowNameInput ] = useState(true)

  useEffect(() => {
    if (visible) {
      form.setFieldsValue({ [type === 'tender' ? 'folderId' : 'id']: id })
      if (type === 'rename') {
        form.setFieldsValue({ name })
      }
      if (type === 'move') {
        setCascader(mapTree(treeObj.children, id, isFolder) as Option[])
      }
      if (type === 'move' || type.indexOf('del') !== -1) {
        setShowNameInput(false)
      } else {
        setShowNameInput(true)
        form.setFieldsValue({ depth: type === 'root' ? -1 : 0 })
      }
    }
  }, [ type, id, visible ])
  return (
    <Modal
      getContainer={false}
      visible={visible}
      confirmLoading={confirmLoading}
      title={modalObj[type].title}
      okText={modalObj[type].okText}
      okButtonProps={{ size: 'small', danger: type.indexOf('del') !== -1 ? true : false }}
      cancelButtonProps={{ size: "small" }}
      cancelText="取消"
      onCancel={onCancel}
      closable={false}
      onOk={() => {
        form.validateFields().then(values => {
          form.resetFields()
          onCreate(values, type)
        }).catch(info => {
          console.error(`Validate Failed:${info}`)
        })
      }}>
      <Form form={form} layout="vertical">
        <Form.Item name={type === 'tender' ? 'folderId' : 'id'} hidden ><Input /></Form.Item>
        {
          showNameInput ? <Form.Item name="name" label={modalObj[type].nameTitle} rules={[ { required: true, message: '请输入文件夹名称!' } ]}>
            <Input allowClear size="small" autoComplete="off" />
          </Form.Item> : null
        }
        {
          showNameInput ? <Form.Item name="depth" hidden><Input /></Form.Item> : null
        }
        {
          type === 'move' ? <Form.Item name="targetFolderId"><TreeSelect treeData={cascader} placeholder={modalObj[type].namePlaceholder} allowClear /></Form.Item> : null
        }
        {
          type.indexOf('del') !== -1 ? <div><p>确认删除「{name}」？</p><p>删除后，数据无法恢复，请谨慎操作。</p></div> : null
        }
      </Form>
    </Modal>
  )
}

function mapTree(treeArr: TenderTree[], id: string, isFolder: boolean) {
  const arr = treeArr?.map((tree: TenderTree) => {
    if (!tree.isfolder) return
    const newTree: Option = { value: tree.id, title: tree.name, disabled: tree.id === id }
    // 要移动的是文件夹
    if (isFolder && tree.isBid) {
      // 该目录下已有标段不可移动文件夹至此目录
      newTree.disabled = true
    }
    if (tree.children?.length) {
      if (tree.children.findIndex(item => item.id === id) !== -1) {
        newTree.disabled = true
      }
      newTree.children = mapTree(tree.children, id, isFolder) as Option[]
    }
    return newTree
  }).filter(item => item) // filter过滤滤掉map可能产生的underfined item
  return arr
}

export default ModalForm
