import Authorization from '@/components/Authorization'
import FileModal from '@/components/FileModal'
import Header from '@/components/Header'
import Slot from '@/components/Header/slot'
import RuleModal from '@/components/RuleModal'
import SvgIcon from '@/components/SvgIcon'
import { tenderStore } from '@/store/mobx'
import { iCreateSafe, iSafeList } from '@/types/safe'
import { safeStatus } from '@/utils/common/constStatus'
import { useListModal } from '@/hooks/tableHooks'
import consts from '@/utils/consts'
import { dayjsFormat } from '@/utils/util'
import { Button, message, Table } from 'antd'
import { ColumnsType } from 'antd/lib/table'
import React, { useEffect, useState } from 'react'
import { Link } from 'react-router-dom'
import { apiCreateSafe, apiSafeList, apiSaveRule } from './api'
import AddModel from './modal'


const SafeList: React.FC<{}> = () => {
  const [ state, setState ] = useListModal(consts.DATA_TYPE.SAFE)
  const [page, setPage] = useState({
    pageNo: 1,
    pageSize: consts.PAGE_SIZE
  })
  useEffect(() => {
    initData()
  }, [])
  const columns: ColumnsType<iSafeList> = [
    {
      title: '序号',
      align: 'center',
      width: '5%',
      // eslint-disable-next-line react/display-name
      render: (_: string, record: any, i: number) => {
        return <span>{(page.pageNo - 1) * page.pageSize + i + 1}</span>
      }
    },
    {
      title: '编号',
      dataIndex: 'code',
      width: '10%',
      // eslint-disable-next-line react/display-name
      render: (text: string, record) => {
        return <Link to={{ pathname: '/console/safe/content/detail/info', state: { id: record.id } }}>{text}</Link>
      }
    },
    {
      title: '检查项目',
      dataIndex: 'inspection',
      width: '10%'
    },
    {
      title: '现场检查情况',
      dataIndex: 'inspectionDetail',
      width: '15%'
    },
    {
      title: '处理要求及措施',
      dataIndex: 'demand',
      width: '15%'
    },
    {
      title: '检查日期',
      dataIndex: 'createTime',
      width: '10%',
      // eslint-disable-next-line react/display-name
      render: (text: string) => {
        return <span>{dayjsFormat(text, 'YYYY-MM-DD')}</span>
      }
    },
    {
      title: '检查人',
      dataIndex: 'auditName',
      width: '15%'
    },
    {
      title: '附件',
      dataIndex: 'fileCounts',
      width: '5%',
      // eslint-disable-next-line react/display-name
      render: (text: string, record) => {
        return (
          <span className="pi-pointer" onClick={() => setState({ ...state, fileModal: { ...state.fileModal, dataId: record.id, visible: true } })}>
            <SvgIcon type="xxh-paperclip1" style={{ color: '#007bff' }} /> {text}
          </span>
        )
      }
    },
    {
      title: '状态',
      dataIndex: 'status',
      width: '10%',
      // eslint-disable-next-line react/display-name
      render: (statu: number) => {
        return <span className={safeStatus[statu].className}>{safeStatus[statu].text}</span>
      }
    }
  ]

  const initData = (pageNo: number = 1, pageSize: number = consts.PAGE_SIZE) => {
    setPage({...page, pageNo, pageSize})
    const initModal = {
      ruleModal: { loading: false, visible: false },
      addModal: { loading: false, visible: false }
    }
    apiSafeList(tenderStore.bid, pageNo, pageSize)
      .then(({ code = -1, data = [], total = 0 }) => {
        if (code === consts.RET_CODE.SUCCESS) {
          setState({ ...state, dataList: data, total, ...initModal })
        }
      })
      .catch(err => {
        console.log(err)
      })
  }

  const onRuleCreate = async (ruleValue: any) => {
    setState({ ...state, ruleModal: { ...state.ruleModal, loading: true } })
    const { code = -1 } = await apiSaveRule({ bidsectionId: tenderStore.bid, type: 'safe_rule', rule: ruleValue })
    if (code === consts.RET_CODE.SUCCESS) {
      initData()
      return message.success('规则更改成功')
    } else {
      setState({ ...state, ruleModal: { ...state.ruleModal, loading: false } })
    }
  }
  const onAddCreate = async (payload: iCreateSafe) => {
    setState({ ...state, addModal: { ...state.addModal, loading: true } })
    const createTime = dayjsFormat(payload.createTime)
    const { code = -1 } = await apiCreateSafe({ ...payload, createTime })
    if (code === consts.RET_CODE.SUCCESS) {
      initData()
      return message.success('添加新的巡检记录成功')
    } else {
      setState({ ...state, addModal: { ...state.addModal, loading: false } })
    }
  }

  return (
    <div className="wrap-contaniner">
      <Header title="安全巡检">
        <Slot position="right">
          <Button
            type="ghost"
            size="small"
            icon={<SvgIcon type="xxh-cog" />}
            className="pi-mg-right-3"
            style={{ color: '#007bff' }}
            onClick={() => setState({ ...state, ruleModal: { ...state.ruleModal, visible: true } })}>
            设置
            </Button>

          <Authorization type="safe" auth="add">
            <Button type="primary" size="small" onClick={() => setState({ ...state, addModal: { ...state.addModal, visible: true } })}>
              新建巡检
            </Button>
          </Authorization>
        </Slot>
      </Header>
      <Table
        dataSource={state.dataList}
        columns={columns}
        pagination={{
          hideOnSinglePage: true,
          size: 'small',
          current: page.pageNo,
          pageSize: page.pageSize,
          onChange: (page, pageSize) => initData(page, pageSize),
          total: state.total
        }}
        rowKey={record => record.id}
        bordered />
      <RuleModal
        type={consts.RULE.SAFE}
        title="安全巡检编号设置"
        visible={state.ruleModal.visible}
        onCreate={onRuleCreate}
        loading={state.ruleModal.loading}
        onCancel={() => setState({ ...state, ruleModal: { ...state.ruleModal, visible: false } })} />
      <AddModel visible={state.addModal.visible} onCreate={onAddCreate} loading={state.addModal.loading} onCancel={() => setState({ ...state, addModal: { ...state.addModal, visible: false } })} />
      <FileModal visible={state.fileModal.visible} dataType={state.fileModal.dataType} dataId={state.fileModal.dataId} onCancel={() => setState({ ...state, fileModal: { ...state.fileModal, visible: false } })} />
    </div>
  )
}

export default SafeList
